# If any of the following macros should be set otherwise,
# you can wrap any of them with the following conditions:
# - %%if 0%%{centos} == 7
# - %%if 0%%{?rhel} == 7
# - %%if 0%%{?fedora} == 23
# Or just test for particular distribution:
# - %%if 0%%{centos}
# - %%if 0%%{?rhel}
# - %%if 0%%{?fedora}
#
# Be aware, on centos, both %%rhel and %%centos are set. If you want to test
# rhel specific macros, you can use %%if 0%%{?rhel} && 0%%{?centos} == 0 condition.
# (Don't forget to replace double percentage symbol with single one in order to apply a condition)

# Generate devel rpm
%global with_devel 1
# Build project from bundled dependencies
%global with_bundled 0
# Build with debug info rpm
%global with_debug 1
# Run tests in check section
%global with_check 0
# Generate unit-test rpm
%global with_unit_test 1

%if 0%{?with_debug}
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%if ! 0%{?gobuild:1}
%define gobuild(o:) go build -ldflags "${LDFLAGS:-} -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n')" -a -v -x %{?**};
%endif

%global provider        github
%global provider_tld    com
%global project         minio
%global repo            mc
# https://github.com/minio/mc
%global provider_prefix %{provider}.%{provider_tld}/%{project}/%{repo}
%global import_path     %{provider_prefix}
%global commit          cca8c10f338d65aa6d0b626b3950a58f74216b6f
%global shortcommit     %(c=%{commit}; echo ${c:0:7})

Name:           golang-%{provider}-%{project}-%{repo}
Version:        0
Release:        0.1.git%{shortcommit}%{?dist}
Summary:        Minio client replacement for common filesystem *nix commands
License:        ASL 2.0
URL:            https://%{provider_prefix}
Source0:        https://%{provider_prefix}/archive/%{commit}/%{repo}-%{shortcommit}.tar.gz

# e.g. el6 has ppc64 arch without gcc-go, so EA tag is required
ExclusiveArch:  %{?go_arches:%{go_arches}}%{!?go_arches:%{ix86} x86_64 aarch64 %{arm}}
# If go_compiler is not set to 1, there is no virtual provide. Use golang instead.
BuildRequires:  %{?go_compiler:compiler(go-compiler)}%{!?go_compiler:golang}

%if ! 0%{?with_bundled}
# Remaining dependencies not included in main packages
BuildRequires: golang(github.com/rjeczalik/notify)
BuildRequires: golang(golang.org/x/text/unicode/norm)
BuildRequires: golang(github.com/cheggaaa/pb)
BuildRequires: golang(github.com/minio/minio-go/pkg/policy)
BuildRequires: golang(github.com/minio/go-homedir)
BuildRequires: golang(github.com/minio/cli)
# BuildRequires: golang(github.com/minio/minio/pkg/probe)
BuildRequires: golang(github.com/fatih/color)
BuildRequires: golang(github.com/minio/minio/pkg/madmin)
BuildRequires: golang(github.com/hashicorp/go-version)
BuildRequires: golang(github.com/pkg/profile)
BuildRequires: golang(github.com/minio/minio-go/pkg/s3utils)
BuildRequires: golang(github.com/mattn/go-colorable)
BuildRequires: golang(github.com/mattn/go-isatty)
BuildRequires: golang(github.com/dustin/go-humanize)
BuildRequires: golang(github.com/minio/minio-go)
BuildRequires: golang(github.com/minio/minio/pkg/quick)
%endif

%description
%{summary}

%if 0%{?with_devel}
%package devel
Summary:       %{summary}
BuildArch:     noarch

%if 0%{?with_check} && ! 0%{?with_bundled}
BuildRequires: golang(github.com/cheggaaa/pb)
BuildRequires: golang(github.com/dustin/go-humanize)
BuildRequires: golang(github.com/fatih/color)
BuildRequires: golang(github.com/hashicorp/go-version)
BuildRequires: golang(github.com/mattn/go-colorable)
BuildRequires: golang(github.com/mattn/go-isatty)
BuildRequires: golang(github.com/minio/cli)
BuildRequires: golang(github.com/minio/go-homedir)
BuildRequires: golang(github.com/minio/minio-go)
BuildRequires: golang(github.com/minio/minio-go/pkg/policy)
BuildRequires: golang(github.com/minio/minio-go/pkg/s3utils)
BuildRequires: golang(github.com/minio/minio/pkg/madmin)
#BuildRequires: golang(github.com/minio/minio/pkg/probe)
BuildRequires: golang(github.com/minio/minio/pkg/quick)
BuildRequires: golang(github.com/pkg/profile)
BuildRequires: golang(github.com/rjeczalik/notify)
BuildRequires: golang(golang.org/x/text/unicode/norm)
%endif

Requires:      golang(github.com/cheggaaa/pb)
Requires:      golang(github.com/dustin/go-humanize)
Requires:      golang(github.com/fatih/color)
Requires:      golang(github.com/hashicorp/go-version)
Requires:      golang(github.com/mattn/go-colorable)
Requires:      golang(github.com/mattn/go-isatty)
Requires:      golang(github.com/minio/cli)
Requires:      golang(github.com/minio/go-homedir)
Requires:      golang(github.com/minio/minio-go)
Requires:      golang(github.com/minio/minio-go/pkg/policy)
Requires:      golang(github.com/minio/minio-go/pkg/s3utils)
Requires:      golang(github.com/minio/minio/pkg/madmin)
#Requires:      golang(github.com/minio/minio/pkg/probe)
Requires:      golang(github.com/minio/minio/pkg/quick)
Requires:      golang(github.com/pkg/profile)
Requires:      golang(github.com/rjeczalik/notify)
Requires:      golang(golang.org/x/text/unicode/norm)

Provides:      golang(%{import_path}/cmd) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/console) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/hookreader) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/httptracer) = %{version}-%{release}
Provides:      golang(%{import_path}/pkg/ioutils) = %{version}-%{release}

%description devel
%{summary}

This package contains library source intended for
building other packages which use import path with
%{import_path} prefix.
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%package unit-test-devel
Summary:         Unit tests for %{name} package
%if 0%{?with_check}
#Here comes all BuildRequires: PACKAGE the unit tests
#in %%check section need for running
%endif

# test subpackage tests code from devel subpackage
Requires:        %{name}-devel = %{version}-%{release}

%if 0%{?with_check} && ! 0%{?with_bundled}
BuildRequires: golang(gopkg.in/check.v1)
%endif

Requires:      golang(gopkg.in/check.v1)

%description unit-test-devel
%{summary}

This package contains unit tests for project
providing packages with %{import_path} prefix.
%endif

%prep
%setup -q -n %{repo}-%{commit}

%build
mkdir -p src/%{provider}.%{provider_tld}/%{project}
ln -s ../../../ src/%{import_path}

%if ! 0%{?with_bundled}
export GOPATH=$(pwd):%{gopath}
%else
# No dependency directories so far
export GOPATH=$(pwd):%{gopath}
%endif

%gobuild -o bin/mc %{import_path}/
#%gobuild -o bin/buildscripts %{import_path}/buildscripts

%install
install -d -p %{buildroot}%{_bindir}
install -p -m 0755 bin/mc %{buildroot}%{_bindir}
#install -p -m 0755 bin/buildscripts %{buildroot}%{_bindir}

# source codes for building projects
%if 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
echo "%%dir %%{gopath}/src/%%{import_path}/." >> devel.file-list
# find all *.go but no *_test.go files and generate devel.file-list
for file in $(find . \( -iname "*.go" -or -iname "*.s" \) \! -iname "*_test.go" | grep -v "vendor") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

# testing files for this project
%if 0%{?with_unit_test} && 0%{?with_devel}
install -d -p %{buildroot}/%{gopath}/src/%{import_path}/
# find all *_test.go files and generate unit-test-devel.file-list
for file in $(find . -iname "*_test.go" | grep -v "vendor") ; do
    dirprefix=$(dirname $file)
    install -d -p %{buildroot}/%{gopath}/src/%{import_path}/$dirprefix
    cp -pav $file %{buildroot}/%{gopath}/src/%{import_path}/$file
    echo "%%{gopath}/src/%%{import_path}/$file" >> unit-test-devel.file-list

    while [ "$dirprefix" != "." ]; do
        echo "%%dir %%{gopath}/src/%%{import_path}/$dirprefix" >> devel.file-list
        dirprefix=$(dirname $dirprefix)
    done
done
%endif

%if 0%{?with_devel}
sort -u -o devel.file-list devel.file-list
%endif

%check
%if 0%{?with_check} && 0%{?with_unit_test} && 0%{?with_devel}
%if ! 0%{?with_bundled}
export GOPATH=%{buildroot}/%{gopath}:%{gopath}
%else
# Since we aren't packaging up the vendor directory we need to link
# back to it somehow. Hack it up so that we can add the vendor
# directory from BUILD dir as a gopath to be searched when executing
# tests from the BUILDROOT dir.
ln -s ./ ./vendor/src # ./vendor/src -> ./vendor

export GOPATH=%{buildroot}/%{gopath}:$(pwd)/vendor:%{gopath}
%endif

%if ! 0%{?gotest:1}
%global gotest go test
%endif

%gotest %{import_path}/cmd
%gotest %{import_path}/pkg/console
%gotest %{import_path}/pkg/hookreader
%gotest %{import_path}/pkg/httptracer
%gotest %{import_path}/pkg/ioutils
%endif

#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files
%license LICENSE
%doc README.md CONTRIBUTING.md
%{_bindir}/mc
#%{_bindir}/buildscripts

%if 0%{?with_devel}
%files devel -f devel.file-list
%license LICENSE
%doc README.md CONTRIBUTING.md
%dir %{gopath}/src/%{provider}.%{provider_tld}/%{project}
%endif

%if 0%{?with_unit_test} && 0%{?with_devel}
%files unit-test-devel -f unit-test-devel.file-list
%license LICENSE
%doc README.md CONTRIBUTING.md
%endif

%changelog
* Sat Feb 04 2017 Tom Callaway <spot@fedoraproject.org> - 0-0.1.gitcca8c10
- First package for Fedora

